/**
  *  MultiImagePhotometry_ is an ImageJ plugin to find & measure stars in a CCD image.
  *  This version opens a text file containing a list of image files and processes all.
  *  Uses class ApPhot, a subclass of StarField.
  *  Glen Cowan, RHUL Physics Department
  *  November, 2004
  */

import ij.*;
import ij.process.*;
import ij.gui.*;
import ij.plugin.*;
import ij.io.*;
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.math.*;
import java.io.*;

public class MultiImagePhotometry_ implements PlugIn {

  static String title = "Aperture Photometry";
  static boolean createWindow = true;

  public void run(String arg) {

    Vector fileVec = SFUtil.getFileVec();
    int numFiles = fileVec.size();
    for (int file=0; file<numFiles; file++){
      String inFile = (String)fileVec.elementAt(file);
    }

    double starThreshold = IJ.getNumber("Enter star threshold (ADU)", 10000);
    SFPar.setStarThreshold(starThreshold);
    IJ.write ("star threshold = " + starThreshold);
    double gain = IJ.getNumber("Enter CCD gain (e-/ADU)", 2.3);
    SFPar.setGain(gain);
    IJ.write ("gain = " + gain);
    double apRadius = 0;         // set below after minimum star separation known

// Loop over input files from fileVec

    for (int file=0; file<numFiles; file++){

// Open file and create image

      String inFile = (String)fileVec.elementAt(file);
      Opener o = new Opener();
      ImagePlus imp = o.openImage(inFile);     // inFile is complete path
      ImageProcessor ip = imp.getProcessor();

//  Create ApPhot object from image, find clusters and peaks

      ApPhot ap = new ApPhot(starThreshold);
      SFUtil.setStarFieldFromIJ(ap, imp, ip);    // initialize from current image
      if ( file == 0 ){
        int nx = ip.getWidth();
        int ny = ip.getHeight();
        IJ.write ("image width  = " + nx);
        IJ.write ("image height = " + ny);
        IJ.write ("total pixels = " + nx*ny);
      }

      ap.fillClusterVec();              // find clusters of pixels over threshold
      ap.trimClusterVec(3);             // eliminate clusters below minimum size
      ap.findPeaks();                   // find local maxima in clusters
      ap.checkPeaks();

// For now turn off cluster map -- eats up too much memory
//      SFUtil.showClusterMap(ap);       // display clusters in a new RGB image

// From clusters form stars, set apertures, measure flux, etc.

      ap.fillStarVec();
      double dMin = ap.getMinDist();
      System.out.println("Smallest distance between star pair = " + dMin);
      if ( file == 0 || apRadius <= 0 ){
        apRadius = IJ.getNumber("Enter signal aperture radius", dMin/2);
        SFPar.setApRadius(apRadius);
        IJ.write("aperture radius = " + apRadius);
      } 
      double bckRInner = 1.5*apRadius;
      double bckROuter = 2.5*apRadius;
      ap.fillApVec(apRadius, bckRInner, bckROuter);
      SFUtil.showStarMap(ap);          // display stars in a new RGB image
      ap.measureAps();
      ap.subtractBck();

      IJ.write(" ");
      IJ.write("----------------------------------------------------------------");
      IJ.write("Results for " + inFile);


      Vector starVec = ap.getStarVec();
      IJ.write("starVec: i, x, y, nStar, sigmaNStar...");
      for (int i=0; i<starVec.size(); i++){
        Star s = (Star)starVec.elementAt(i);
        double x   = s.getX();
        double y   = s.getY();
        double nStar = s.getNStar();
        double sigmaNStar = s.getSigmaNStar();
        IJ.write(i + "  " + x + "  " + y + "  " + nStar + "  " + sigmaNStar);
      }

// Compute magnitude difference between star 0 (should be brightest) and others

      int numStars = (ap.getStarVec()).size();
      for (int i=1; i<numStars; i++){
        double mij = ap.deltaM(0,i);
        double sigmaMij = ap.sigmaDeltaM(0,i);
        IJ.write("m0" + " - m" + i + " = " + mij + " +- " + sigmaMij);
      }

    }       // end of loop over input files

  }          // end of run

}            // end of class MultiImagePhotometry_
